/*
 * Avkdning av en "Upprepningskodad" meddelande.  Given ett
 * repetitionsvärde ”r”, mottagaren kan korrigera eventuella
 * transmissionsfel genom att räkna upp majoriteten (för varje
 * original bit) för att härleda innehållet av det originella
 * meddelandet. 
 */

class RepetitionDecode{

    private static int r=0;

    /* Går igenom argumenterna och kastar ett "ParsingException" ifa
     * inputen har ett dåligt format (inget eller dåligt 
     * repetitionsvärde, antal inputs icke delbart med repetitionsvärdet,
     * input värden är inte 0s eller 1s). Annars, initialiserar r till 
     * repetitionsvärdet och returnerar värderna i form av en int[] 
     * där sorleken är n med n%r == 0.
     */
    private static int[] parse(String[] args) throws ParsingException{
	int[] values;
	try{	    
	    r = Integer.parseInt(args[0]);
	    if((r <= 0) || ((r%2) == 0) || ((args.length - 1) % r) != 0)
		throw new ParsingException("Invalid input");
	    values = new int[(args.length - 1)];
	    for(int i=1; i < args.length; i++){
		values[i-1] = Integer.parseInt(args[i]);
		if(!(values[i-1]==0 || values[i-1]==1))
		    throw new ParsingException("Invalid input");
	    }
	}catch(NumberFormatException e){
	    throw new ParsingException("Invalid input");
	}catch(ArrayIndexOutOfBoundsException e){
	    throw new ParsingException("Invalid input");
	}
	return values;
    }

    /* Får n*r värden i form av en int[] och returnerar 
     * n stycken majoritet resultat enligt protokollet.
     */

    private static int[] compute(int[] values){
	int[] result;
        result = new int[values.length / r];
	for(int i=0; i< result.length; i++){
	    int s=0;
	    for(int j=0; j< r; j++)
		s+= values[(i*r)+j];
	    if(s <= r/2)
		result[i]=0;
	    else result[i]=1;
	}
	return result;
    }

    /* Skriver ut resultet som är en int[]      
     */
    private static void printResult(int[] result){	
	System.out.print("Decoded message: ");
	for(int i=0; i<result.length; i++)
	    System.out.print(result[i] + " ");
	System.out.println();
    }
    
    /* Givet ett repetitionsvärde och ett antal värden,
     * kollar upp om inputen är välformat, räknar upp majoriteten, 
     * skriver ut resultatet.
     */
    public static void main(String[] args){

	try{
	    int[] values = parse(args);
	    int result[] = compute(values);
	    printResult(result);	    
	}catch(ParsingException e){
	    System.out.println("Syntax: RepetitionDecode r b1 b2 ... bn");
	    System.out.println("With  : r satisfies (r%2 == 1) and (r >= 1)");
	    System.out.println("n satisfies (n%r == 0) and (n >= 0), and");
	    System.out.println("bi satisfies (0 <= bi <= 1) for each i");
	    System.out.println("E.g.  : RepetitionDecode 3 1 0 1 1 0 0");
	}
	
    }

}
